from org.maemo.hermes.engine.friend import Friend
from org.maemo.hermes.engine.contact import Contact
import evolution

class Hermes:
    """Encapsulate the process of syncing online friends' information with the
       Evolution contacts' database. This should be used as follows:
       
         * Initialise, passing in a GUI callback.
         * Call Syncjob.
         * Retrieve information on changes effected.
         * Call update_contact to enact manual mapping.
         
       Copyright (c) Andrew Flegg <andrew@bleb.org> 2010.
       Released under the Artistic Licence."""

    
    # -----------------------------------------------------------------------
    def __init__(self, services, gui_progress):
        """Constructor. Passed a list of services, and a callback method
           which must implement the following API.
                              
             progress(i, j) - the application is currently processing friend 'i' of
                              'j'. Should be used to provide the user a progress bar.
        """
        
        # -- These fields are currently part of the API...
        #
        self.updated = []
        self.matched = []
        self.unmatched = []
        self.friends = {}
        self.addresses = None
        
        # -- Other initialisation...
        #
        self._services = services
        self._progress = gui_progress
    
    
    # -----------------------------------------------------------------------
    def run(self, resync=False):
        """Load information on the authenticated user's friends. Synchronise Facebook
           profiles to contact database. If resync is false, no existing information
           will be overwritten."""

        class FakeContact():
            def get_name(self):
                return "Fredrik Wendt"
            def get_emails(self):
                return ["fredrik@wendt.se", "maemohermes@wendt.se"]
            def get_photo(self):
                return None
            def get_mapped_to(self):
                return set(["facebook", "gravatar"])
        self.matched = [FakeContact()]
 
#        self._sync_job = SyncJob(services, [FakeContact()], self.progress)
#        self._sync_job.run()
#        self._sync_job.get_unmatched_friends()
#        self._sync_job.get_updated_contacts()
#        self._sync_job.get_matched_contacts()
        pass
    
    
    # -----------------------------------------------------------------------
    def run_alt(self, overwrite_existing_fields=False):
        contacts = []
        book = evolution.ebook.open_addressbook('default')
        for contact in book.get_all_contacts():
            contacts.append(Contact(book, contact))

        # warm up
        for service in self._services:
            for contact in contacts:
                service.pre_process_contact(contact)
                
        # fetch data
        for service in self._services:
            service.process_friends()
        
        # combine results into one friend
        for contact in contacts:
            result = Friend()
            for service in self._services:
                friend = service.process_contact(contact)
                if (friend):
                    friend.decorate(result)
            
            self.update_contact(friend, overwrite_existing_fields)
        

    # -----------------------------------------------------------------------
    def update_contact(self, contact, friend, resync=False):
        """Update the given contact with information from the given friend."""
        
        print "updating contact ", contact, " with friend ", friend

    def create_contact_from_friend(self, friend):
        contact = evolution.ebook.EContact()
        contact.props.full_name = friend['name']
        contact.props.given_name = friend['first_name']
        contact.props.family_name = friend['last_name']
        
        self.update_contact(contact, friend)
        
        self.addresses.add_contact(contact)
        self.updated.append(contact)
        self.addresses.commit_contact(contact)
        
        print "Created [%s]" % (contact.get_name())
        self.matched.append(contact)


    def commit(self):
        self.store.close()
