from friend import Friend

class Service:
    """The notional `Service' for a provider. This is responsible for communicating
       with the backend service and enhancing contacts.
       
       Copyright (c) Andrew Flegg <andrew@bleb.org> 2010.
       Released under the Artistic Licence."""
       
    def __init__(self):
        """Should make initial calls to the service and retrieve a list of friends."""


    # -----------------------------------------------------------------------
    def get_name(self):
        """Should return the same name as the provider class - debugging only - REMOVE/FIXME"""
        
        return None

    
    # -----------------------------------------------------------------------
    def pre_process_contact(self, contact):
        """If the contact have an URL (or similar) that proves an existing matching
           to a friend on this service, then this should be remembered to avoid
           name collision/mapping the same friend to other contacts with the same 
           name."""
         
        return None

    
    def process_friends(self):
        """Called once to signal that it's time to get all friends' data."""
        
        pass
    
    
    # -----------------------------------------------------------------------
    def process_contact(self, contact):
        """Called for each contact in the address book. If the contact can be 
           matched to a Friend, than return the Friend object or None."""
           
        pass
    
    
    # -----------------------------------------------------------------------
    def get_unmatched_friends(self):
        """Return a list of friends not matched to a contact, or 'None' if manual mapping
           is not supported."""
         
        return None


    # -----------------------------------------------------------------------
    def finalise(self, updated, overwrite = False):
        """Once all contacts have been processed, allows for any tidy-up/additional
           enrichment. If any contacts are updated at this stage, 'updated' should
           be added to."""
           
        pass


    # -----------------------------------------------------------------------
    ###DEPRECATED
    def _create_friend(self, name):
        """Used to create a Friend object for a contact"""
        
        return Friend(name)
